@@ -6,7 +6,7 @@
 ;; URL: http://github.com/s-kostyaev/ellama
 ;; Keywords: help local tools
 ;; Package-Requires: ((emacs "28.1") (llm "0.24.0") (plz "0.8") (transient "0.7") (compat "29.1") (yaml "1.2.3"))
-;; Version: 1.11.1
+;; Version: 1.12.9
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 ;; Created: 8th Oct 2023
 
@@ -1217,9 +1217,10 @@ Otherwire return current active session."
   "Search for AGENTS.md file from current directory up to project root.
 Returns the full path to AGENTS.md if found, or nil if not found."
   (let* ((current-dir (file-name-directory (expand-file-name default-directory)))
-         (project-root
-	  (file-name-directory (expand-file-name
-				(or (project-root (project-current)) current-dir))))
+         (project-root (or (and (ellama-tools-project-root-tool)
+				(file-name-directory (expand-file-name
+						      (ellama-tools-project-root-tool))))
+			   current-dir))
          found-path)
     ;; Walk up from current directory to project root
     (while (and (not found-path)
@@ -1242,9 +1243,10 @@ Returns the full path to AGENTS.md if found, or nil if not found."
 	   (buffer-string))))
       ""))
 
-(defun ellama-get-system-message ()
-  "Return the effective system message, including dynamically scanned skills."
-  (let ((msg (concat (or ellama-global-system "")
+(defun ellama-get-system-message (&optional msg)
+  "Return the effective system message, including dynamically scanned skills.
+MSG is a text that will be included in the resulting system message."
+  (let ((msg (concat (or msg ellama-global-system "")
 		     (ellama-get-agents-md)
 		     (ellama-skills-generate-prompt))))
     (when (not (string= msg ""))
@@ -1503,6 +1505,8 @@ in.  Default value is (current-buffer).
 
 :point POINT -- POINT is the point in buffer to insert ellama reply at.
 
+:tools LIST -- LIST of enabled in the current session tools.
+
 :filter FILTER -- FILTER is a function that's applied to (partial) response
 strings before they're inserted into the BUFFER.
 
@@ -1547,8 +1551,13 @@ failure (with BUFFER current).
 		      (error "Error calling the LLM: %s" msg))))
 	 (donecb (or (plist-get args :on-done) #'ignore))
 	 (prompt-with-ctx (ellama-context-prompt-with-context prompt))
-	 (system (or (plist-get args :system)
-		     (ellama-get-system-message)))
+	 (system (ellama-get-system-message (plist-get args :system)))
+	 (session-tools (and session
+                             (ellama-session-extra session)
+                             (plist-get (ellama-session-extra session) :tools)))
+	 (tools (or session-tools
+		    (plist-get args :tools)
+		    ellama-tools-enabled))
 	 (llm-prompt (if session
 			 (if (llm-chat-prompt-p (ellama-session-prompt session))
 			     (progn
@@ -1556,7 +1565,7 @@ failure (with BUFFER current).
 				(ellama-session-prompt session)
 				prompt-with-ctx)
 			       (setf (llm-chat-prompt-tools (ellama-session-prompt session))
-				     ellama-tools-enabled)
+				     tools)
 			       (when system
 				 (llm-chat-prompt-append-response
 				  (ellama-session-prompt session)
@@ -1564,9 +1573,9 @@ failure (with BUFFER current).
 			       (ellama-session-prompt session))
 			   (setf (ellama-session-prompt session)
 				 (llm-make-chat-prompt prompt-with-ctx :context system
-						       :tools ellama-tools-enabled)))
+						       :tools tools)))
 		       (llm-make-chat-prompt prompt-with-ctx :context system
-					     :tools ellama-tools-enabled))))
+					     :tools tools))))
     (with-current-buffer reasoning-buffer
       (org-mode))
     (with-current-buffer buffer
